/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#pragma once

#include <QSet>
#include <QString>

#include "Message.h"

/**
 * @brief A filter specifies a topic and an optional list of fields.
 *
 * The topic can contain wildcard symbols * and ? as last token. For example:
 * - "*" and "?" match with all topics
 * - "Prefix/?" matches with "Prefix", "Prefix/A" and "Prefix/A/B"
 * - "Prefix/any" matches with "Prefix/A" and "Prefix/A/B"
 *
 * The difference between * and ? is that the latter also matches when only the
 * prefix is present.
 */
class Filter
{
public:
    Filter();
    explicit Filter(Topic topic);
    Filter(Topic topic, QSet<QString> fields);

    bool operator==(const Filter& toCompare) const;
    bool operator!=(const Filter& toCompare) const;
    bool operator<(const Filter& toCompare) const;

    void setTopic(Topic topic);
    Topic getTopic() const;

    void addField(QString field);
    bool removeField(QString field);
    QSet<QString> getFields() const;

    /**
     * @brief Checks if the message topic is matched by the filter.
     */
    bool match(const Message& message) const;

    /**
     * @brief Keeps only the fields specified by the filter.
     *
     * If the filter has no fields then the messages is not modified.
     */
    bool filterMessage(Message& message) const;

    QString toString() const;
    QString toPrettyString() const;
    static Filter fromString(QString string);

private:
    Topic topic;
    QSet<QString> fields;
};
